export class NuxtMultiCacheRouteCacheHelper {
  /**
   * The collected cache tags.
   */
  tags;
  /**
   * Indicates if the route should be cacheable.
   */
  cacheable;
  /**
   * The maximum age.
   */
  maxAge;
  constructor() {
    this.tags = [];
    this.cacheable = null;
    this.maxAge = null;
  }
  /**
   * Add cache tags for this route.
   */
  addTags(tags = []) {
    this.tags.push(...tags);
    return this;
  }
  /**
   * Mark this route as cacheable.
   *
   * The initial value is null and this method only changes the value if it is
   * null. This means that once it's set to uncacheable, there is no way to
   * change it back.
   */
  setCacheable() {
    if (this.cacheable === null) {
      this.cacheable = true;
    }
    return this;
  }
  /**
   * Mark the route as uncacheable.
   *
   * After that there is no way to make it cacheable again.
   */
  setUncacheable() {
    this.cacheable = false;
    return this;
  }
  /**
   * Set the max age in seconds.
   *
   * The value is only set if it's smaller than the current max age or if it
   * hasn't been set yet. The initial value is `null`.
   *
   * You can always directly set the maxAge property on this object.
   */
  setMaxAge(maxAge = 0) {
    if (!this.maxAge || maxAge < this.maxAge) {
      this.maxAge = maxAge;
    }
    return this;
  }
}
