import { defineEventHandler } from "h3";
import { format } from "@tusbar/cache-control";
import { encodeRouteCacheItem } from "../helpers/cacheItem.mjs";
import {
  getMultiCacheCDNHelper,
  getMultiCacheContext,
  getMultiCacheRouteHelper,
  getCacheKeyWithPrefix,
  encodeRouteCacheKey
} from "./../helpers/server.mjs";
import serverOptions from "#multi-cache-server-options";
import { useRuntimeConfig } from "#imports";
import { logger } from "../helpers/logger.mjs";
export default defineEventHandler((event) => {
  const cdnHelper = getMultiCacheCDNHelper(event);
  const multiCache = getMultiCacheContext(event);
  const runtimeConfig = useRuntimeConfig();
  const debug = runtimeConfig.multiCache.debug;
  if (!cdnHelper && !multiCache?.route) {
    return;
  }
  const response = event.node.res;
  const _end = response.end;
  response.end = function(arg1, arg2, arg3) {
    if (typeof arg1 === "function") {
      return _end.call(event.node.res, arg1);
    }
    const chunk = arg1;
    if (cdnHelper) {
      const cacheTagsValue = cdnHelper._tags.join(" ");
      if (cacheTagsValue) {
        response.setHeader(
          runtimeConfig.multiCache.cdn.cacheTagHeader,
          cacheTagsValue
        );
      }
      const cacheControlValue = format(cdnHelper._control);
      if (cacheControlValue) {
        response.setHeader(
          runtimeConfig.multiCache.cdn.cacheControlHeader,
          cacheControlValue
        );
      }
    }
    if (event.path) {
      const routeHelper = getMultiCacheRouteHelper(event);
      if (routeHelper && routeHelper.cacheable) {
        const multiCache2 = getMultiCacheContext(event);
        if (multiCache2?.route && event.path) {
          const cacheKey = serverOptions?.route?.buildCacheKey ? serverOptions.route.buildCacheKey(event) : getCacheKeyWithPrefix(encodeRouteCacheKey(event.path), event);
          const response2 = event.node.res;
          const expires = routeHelper.maxAge ? Math.round(Date.now() / 1e3) + routeHelper.maxAge : void 0;
          const cacheItem = encodeRouteCacheItem(
            chunk,
            response2.getHeaders(),
            response2.statusCode,
            expires,
            routeHelper.tags
          );
          if (debug) {
            logger.info("Storing route in cache: " + event.path, {
              expires,
              cacheTags: routeHelper.tags,
              statusCode: response2.statusCode
            });
          }
          return multiCache2.route.setItemRaw(cacheKey, cacheItem, { ttl: routeHelper.maxAge }).then(() => {
            return _end.call(event.node.res, arg1, arg2, arg3);
          });
        }
      }
    }
    return _end.call(event.node.res, arg1, arg2, arg3);
  };
});
