import { defineEventHandler, setResponseHeaders } from "h3";
import { decodeRouteCacheItem } from "../helpers/cacheItem.mjs";
import { logger } from "../helpers/logger.mjs";
import {
  getMultiCacheContext,
  getCacheKeyWithPrefix,
  encodeRouteCacheKey
} from "./../helpers/server.mjs";
import serverOptions from "#multi-cache-server-options";
import { useRuntimeConfig } from "#imports";
const { debug } = useRuntimeConfig().multiCache || {};
export default defineEventHandler(async (event) => {
  if (!event.path) {
    return;
  }
  const multiCache = getMultiCacheContext(event);
  if (!multiCache?.route) {
    return;
  }
  try {
    const fullKey = serverOptions?.route?.buildCacheKey ? serverOptions.route.buildCacheKey(event) : getCacheKeyWithPrefix(encodeRouteCacheKey(event.path), event);
    const cachedRaw = await multiCache.route.getItemRaw(fullKey);
    if (cachedRaw && typeof cachedRaw === "string") {
      const decoded = decodeRouteCacheItem(cachedRaw);
      if (decoded) {
        if (decoded.expires) {
          const now = Date.now() / 1e3;
          if (now >= decoded.expires) {
            return;
          }
        }
        if (decoded.headers) {
          setResponseHeaders(event, decoded.headers);
        }
        if (decoded.statusCode) {
          event.node.res.statusCode = decoded.statusCode;
        }
        if (debug) {
          logger.info("Serving cached route for path: " + event.path, {
            fullKey
          });
        }
        return decoded.data;
      }
    }
  } catch (e) {
    if (e instanceof Error) {
      console.debug(e.message);
    }
  }
});
